function h = convex_hull(x)
%
% This function returns the vertices of the convex hull of the set of
% of points in x.  The top part of the hull is ordered by ascending real
% part; the bottom is ordered by descending real part.
%

[mx,nx] = size(x);
if (mx < nx) x = conj(x)'; end;
sx = rsort(x);
nsx = length(sx);
if (nsx <= 0), return, end;


%%% Eliminate "extra" real points 
ireal = find(imag(sx) == 0);
ireal = sort(ireal);
im = length(ireal)-1;
sx(ireal(2:im)) = [];
nsx = length(sx);
if (nsx <= 0), return, end;

%%% Compute the top of the hull
   ht = sx;
   nt = nsx;
   j = 1;
   while j < nt
      %%% Eliminate the points with the same real part as ht(i)
      k = sum( real(ht(j:nt)) == real(ht(j)) ) - 1;
      if k ~= 0
	 [z,l] = max(real(-i*ht(j:j+k)));
   	 ht(j) = ht(j+l-1);
	 ht(j+1:j+k) = [];
	 nt=length(ht);
      end
      %%%Find the next corner of the hull
      if j < nt
         [z,l]= max(angle(ht(j+1:nt) - ht(j)));
         if l > 1
	    ht(j+1:j+l-1) = [];
            nt = length(ht);
         end
      end
      j = j+1;
   end

%%% Compute the bottom of the hull
   hb = sx;
   nb = length(sx);
   j = 1;
   while j < nb
      %%% Eliminate the points with the same real part as hb(i)
      k = sum( real(hb(j:nb)) == real (hb(j)) ) - 1;
      if k ~= 0
	 [z,l] = min(real(-i*hb(j:j+k)));
 	 hb(j) = hb(j+l-1);
	 hb(j+1:j+k) = [];
	 nb=length(hb);
      end
      %%% Find the next corner of the hull
      if j < nb
         [z,l]= min(angle(hb(j+1:nb) - hb(j)));
         if l > 1
	    hb(j+1:j+l-1) = [];
            nb = length(hb);
         end
      end
      j=j+1;
   end

%%% Put the top and bottom together
   if (ht(nt) == hb(nb))
      if (ht(1) == hb(1))
         h = [ht' hb(nb-1:-1:2)']';
      else
         h = [ht' hb(nb-1:-1:1)']';
      end
   else
      h = [ht' hb(nb:-1:1)']';
   end
end
return
