function show_eigs(A, title_string, hull_flag)
%
%  Further modified 11-1-94 to compute the field of values if the
%  matrix is not too large.

%  This function computes and plots the eigenvalues of the matrix A. 
%  It also computes and plots the convex hull if hull_flag is true.
%  The graph is titled with the string "title_string".
%

   if (nargin < 2), title_string = 'Eigenvalues'; end
   if (nargin < 3), hull_flag = (1==1); end
   
   B = full(A);
   EigA = eig(B); 
   ReEigA = real(EigA);
   ImEigA = imag(EigA);

   xmn = min(ReEigA);   xmx = max(ReEigA);
   ymn = min(ImEigA);   ymx = max(ImEigA);
   ymin = min([ymn,-ymx,xmn,-xmx]);
   ymax = max([-ymn,ymx,-xmn,xmx]);
   xmin = min([ymin,xmn,-xmx]);
   xmax = max([ymax,-xmn,xmx]);

   clf; hold off; axis('square');
   axis(1.1*[xmin xmax ymin ymax]); 
   haxis = [xmin 0; xmax 0]; vaxis = [0+i*ymin; 0+i*ymax];
   plot(real(vaxis),imag(vaxis),'-g'); hold on;
   plot(real(haxis),imag(haxis),'-g'); hold on;

   plot(ReEigA, ImEigA, '*b');
   if hull_flag
      ch = convex_hull(EigA);
      if length(ch) > 0, ch = [ch ; ch(1)]; end;
%      plot(real(ch), imag(ch), '-r'); % ``r'' mean red. It doesn't
                                       %show on a BW workstation. 11-1-94.
      plot(real(ch), imag(ch))
   end 
   
   title(title_string);
   axis('normal'); hold off;

%  thmax needed only for the field of values. 
   thmax = 16;   
   nk = 1;
   [f, e] = fv(A, EigA, nk, thmax);%, noplot) % Plot controlled by nargins. 11-1-94.
   

return
