/*------------------------------------------------------------------------
// Copyright (C) 1993,1994: 
// J.C. Meza
// Sandia National Laboratories
// meza@california.sandia.gov
//----------------------------------------------------------------------*/

#include <sys/param.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/times.h>
#include <sys/resource.h>

#ifdef __alpha   /* For DEC Alpha workstations */
#define HZ 100
#endif

#ifdef __t3d    /* For Cray T3D */
#define HZ 1000000
#endif

#ifdef __t3e    /* For Cray T3E */
#define HZ 1000000
#endif

double get_cpu_time()
{
/* ********************************************************************
**
**  Name: get_cpu_time
**
**  Purpose: general purpose CPU timing routine.
**
**  Arguments: none.
**
**  Return Value: user CPU time in (double) seconds.
**
**  Revision History:
**
**  10-May-94 -- TXF; initial development of get_cpu_time ().
**
** *******************************************************************/

    struct tms tms;
/*
**  Begin get_cpu_time.
*/
    times (&tms);
    return ((double) tms.tms_utime / (double) HZ);
/*
**  End get_cpu_time.
*/
}
double get_wall_clock_time()
{
/* ********************************************************************
**
**  Name: get_wall_clock_time
**
**  Purpose: general purpose wall-clock timing routine.
**
**  Arguments: none.
**
**  Return Value: time in (double) seconds since the Epoch.
**
**  Revision History:
**
**  10-May-94 -- TXF; initial development of get_wall_clock_time ().
**
** *******************************************************************/

    struct timeval tp;
/*
**  Begin get_wall_clock_time.
*/
    gettimeofday (&tp, NULL);
    return ((double) tp.tv_sec + ((double) tp.tv_usec / 1.0e06));
/*
**  End get_wall_clock_time.
*/
}
double get_cpu_time_()
{
/* ********************************************************************
**
**  Name: get_cpu_time_
**
**  Purpose: general purpose Fortran callable CPU timing routine.
**
**  Arguments: none.
**
**  Return Value: user CPU time in (double) seconds.
**
**  Revision History:
**
**  16-Aug-95 -- TXF; initial development of get_cpu_time_ ().
**
** *******************************************************************/

    double get_cpu_time();
/*
**  Begin get_cpu_time_.
*/
    return get_cpu_time();
/*
**  End get_cpu_time_.
*/
}
double get_wall_clock_time_()
{
/* ********************************************************************
**
**  Name: get_wall_clock_time_
**
**  Purpose: general purpose wall-clock timing routine.
**
**  Arguments: none.
**
**  Return Value: time in (double) seconds since the Epoch.
**
**  Revision History:
**
**  16-Aug-95 -- TXF; initial development of get_wall_clock_time_ ().
**
** *******************************************************************/

    double get_wall_clock_time();
/*
**  Begin get_wall_clock_time_.
*/
    return get_wall_clock_time();
/*
**  End get_wall_clock_time_.
*/
}
